import { createFilter } from 'rollup-pluginutils';

/**
 * 数据类型判断
 * @param  subject 待判断的数据
 * @param  type    数据类型名字
 * @return         判断结果
 */
function is(subject, type) {
    return Object.prototype.toString.call(subject).substr(8, type.length).toLowerCase() === type;
}

/**
 * 是否是数组
 * @param  subject 待判断的数据
 */
function isObject(subject) {
    return is(subject, "object");
}

/**
 * 是否 undefined
 * @param  subject 待判断的数据
 */
function isUndefined(subject) {
    return is(subject, "undefined");
}

/**
 * 是否是正则
 * @param subject 待判断数据
 */
function isRegexp(subject) {
    return is(subject, "regexp");
}

/**Rollup 文本文件加载插件 */
function stringLoader(opts) {
    if (isUndefined(opts.include)) {
        opts.include = "**/*.html";
    }
    const filter = createFilter(opts.include, opts.exclude);
    return {
        "name": "string",
        transform(sources, id) {
            if (filter(id)) {
                var code = sources;
                if (isObject(opts.replacements) && isRegexp(opts.replacements.reg)) {
                    if (isUndefined(opts.replacements.val)) {
                        opts.replacements.val = "";
                    }
                    code = code.replace(opts.replacements.reg, opts.replacements.val);
                }
                if (opts.minify) {
                    code = code.replace(/\n[\s]+/g, "");
                }
                return {
                    "code": `export default ${JSON.stringify(code)};`,
                    "map": {
                        "mappings": ""
                    }
                };
            }
        }
    };
}

export { stringLoader };
